@@ -24,18 +24,24 @@
 #endif
 
 #include <glib/gstdio.h>
-#include <gnome.h>
 #include <pwd.h>
 #include <gio/gio.h>
-#include <libgnomeui/gnome-thumbnail.h>
+#include <libgnomeui/gnome-desktop-thumbnail.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 #include <pwd.h>
 #include <unistd.h>
 #include <libebook/e-book.h>
+#include <dbus/dbus-glib-bindings.h>
+
+#ifdef HAVE_POLKIT
+#include <polkit-gnome/polkit-gnome.h>
+#endif
 
 #include "e-image-chooser.h"
 #include "gnome-about-me-password.h"
+#include "gnome-about-me-fingerprint.h"
+#include "marshal.h"
 
 #include "capplet-util.h"
 
@@ -49,10 +55,12 @@ typedef struct {
 	EBook    	*book;
 
 	GladeXML 	*dialog;
+	GtkWidget	*enable_fingerprint_button;
+	GtkWidget	*disable_fingerprint_button;
 
 	GdkScreen    	*screen;
 	GtkIconTheme 	*theme;
-	GnomeThumbnailFactory *thumbs;
+	GnomeDesktopThumbnailFactory *thumbs;
 
 	EContactAddress *addr1;
 	EContactAddress *addr2;
@@ -157,6 +165,27 @@ about_me_error (GtkWindow *parent, gchar *str)
 	gtk_widget_destroy (dialog);
 }
 
+#ifdef HAVE_POLKIT
+static GtkWidget *
+create_fingerprint_button (const char *msg, const char *name)
+{
+	GtkWidget *button;
+	PolKitAction *action;
+	PolKitGnomeAction *gaction;
+
+	action = polkit_action_new_from_string_representation ("net.reactivated.fprint.device.enroll");
+	gaction = polkit_gnome_action_new_default (name,
+						   action,
+						   msg,
+						   NULL);
+	polkit_action_unref (action);
+
+	button = polkit_gnome_action_create_button (gaction);
+	g_object_unref (gaction);
+
+	return button;
+}
+#endif /* HAVE_POLKIT */
 
 /********************/
 static void
@@ -621,10 +650,11 @@ about_me_update_photo (GnomeAboutMe *me)
 		/* Update the image in the card */
 		e_contact_set (me->contact, E_CONTACT_PHOTO, NULL);
 
-		/* Update GDM configuration */
-		gnome_config_set_string ("/gdmphotosetup/last/picture", "");
-		gnome_config_set_string ("/gdm/face/picture", "");
-		gnome_config_sync ();
+		file = g_build_filename (g_get_home_dir (), ".face", NULL);
+
+		g_unlink (file);
+
+		g_free (file);
 	}
 
 	about_me_commit (me);
@@ -650,6 +680,9 @@ about_me_load_info (GnomeAboutMe *me)
 	for (i = 0; ids[i].wid != NULL; i++) {
 		about_me_load_string_field (me, ids[i].wid, ids[i].cid, i);
 	}
+
+	set_fingerprint_label (me->enable_fingerprint_button,
+			       me->disable_fingerprint_button);
 }
 
 GtkWidget *
@@ -680,7 +713,7 @@ about_me_update_preview (GtkFileChooser *chooser,
 		GFileInfo *file_info;
 
 		if (!me->thumbs)
-			me->thumbs = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
+			me->thumbs = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
 
 		file = g_file_new_for_uri (uri);
 		file_info = g_file_query_info (file,
@@ -698,7 +731,7 @@ about_me_update_preview (GtkFileChooser *chooser,
 
 				mime_type = g_content_type_get_mime_type (content_type);
 
-				pixbuf = gnome_thumbnail_factory_generate_thumbnail (me->thumbs,
+				pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (me->thumbs,
 										     uri,
 										     mime_type);
 				g_free (mime_type);
@@ -856,6 +889,14 @@ about_me_passwd_clicked_cb (GtkWidget *button, GnomeAboutMe *me)
 	gnome_about_me_password (GTK_WINDOW (WID ("about-me-dialog")));
 }
 
+static void
+about_me_fingerprint_button_clicked_cb (GtkWidget *button, GnomeAboutMe *me)
+{
+	fingerprint_button_clicked (me->dialog,
+				    me->enable_fingerprint_button,
+				    me->disable_fingerprint_button);
+}
+
 static gint
 about_me_setup_dialog (void)
 {
@@ -989,6 +1030,25 @@ about_me_setup_dialog (void)
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (about_me_image_clicked_cb), me);
 
+#ifdef HAVE_POLKIT
+	gtk_widget_destroy (WID ("enable_fingerprint_button"));
+	gtk_widget_destroy (WID ("disable_fingerprint_button"));
+
+	me->enable_fingerprint_button = create_fingerprint_button (_("Enable _Fingerprint Login..."), "enable-action");
+	gtk_container_add (GTK_CONTAINER (WID("buttons_vbox")), me->enable_fingerprint_button);
+
+	me->disable_fingerprint_button = create_fingerprint_button (_("Disable _Fingerprint Login..."), "disable-action");
+	gtk_container_add (GTK_CONTAINER (WID("buttons_vbox")), me->disable_fingerprint_button);
+#else
+	me->enable_fingerprint_button = WID ("enable_fingerprint_button");
+	me->disable_fingerprint_button = WID ("disable_fingerprint_button");
+#endif /* HAVE_POLKIT */
+
+	g_signal_connect (me->enable_fingerprint_button, "clicked",
+			  G_CALLBACK (about_me_fingerprint_button_clicked_cb), me);
+	g_signal_connect (me->disable_fingerprint_button, "clicked",
+			  G_CALLBACK (about_me_fingerprint_button_clicked_cb), me);
+
 	widget = WID ("image-chooser");
 	g_signal_connect (widget, "changed",
 			  G_CALLBACK (about_me_image_changed_cb), me);
@@ -1024,17 +1084,11 @@ about_me_setup_dialog (void)
 int
 main (int argc, char **argv)
 {
-	GnomeProgram *program;
 	int rc = 0;
 
-	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
-
-	program = gnome_program_init ("gnome-about-me", VERSION,
-			LIBGNOMEUI_MODULE, argc, argv,
-			GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
-			NULL);
+	capplet_init (NULL, &argc, &argv);
+	dbus_g_object_register_marshaller (fprintd_marshal_VOID__STRING_BOOLEAN,
+					   G_TYPE_NONE, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INVALID);
 
 	rc = about_me_setup_dialog ();
 
@@ -1042,7 +1096,5 @@ main (int argc, char **argv)
 		gtk_main ();
 	}
 
-	g_object_unref (program);
-
 	return rc;
 }
