@@ -29,8 +29,7 @@
 #include <gio/gio.h>
 #include <string.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-thumbnail.h>
-#define GNOME_DESKTOP_USE_UNSTABLE_API
+#include <libgnomeui/gnome-desktop-thumbnail.h>
 #include <libgnomeui/gnome-bg.h>
 
 enum {
@@ -858,9 +857,9 @@ wp_update_preview (GtkFileChooser *chooser,
 
     if (mime_type)
     {
-      pixbuf = gnome_thumbnail_factory_generate_thumbnail (data->thumb_factory,
-                                                           uri,
-                                                           mime_type);
+      pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (data->thumb_factory,
+								   uri,
+								   mime_type);
     }
 
     if (pixbuf != NULL)
@@ -901,6 +900,13 @@ wp_load_stuffs (void *user_data)
   uri = gconf_client_get_string (data->client,
                                  WP_FILE_KEY,
                                  NULL);
+
+  if (uri && *uri == '\0')
+  {
+    g_free (uri);
+    uri = NULL;
+  }
+  
   if (uri == NULL)
     uri = g_strdup ("(none)");
 
