@@ -21,17 +21,28 @@
 
 #include <string.h>
 #include <pango/pango.h>
+#include <glib/gi18n.h>
 
 #include "theme-util.h"
 #include "gtkrc-utils.h"
 #include "gconf-property-editor.h"
 #include "theme-thumbnail.h"
+#include "capplet-util.h"
+#include "gedit-message-area.h"
 
 typedef void (* ThumbnailGenFunc) (void               *type,
 				   ThemeThumbnailFunc  theme,
 				   AppearanceData     *data,
 				   GDestroyNotify     *destroy);
 
+typedef struct {
+  AppearanceData *data;
+  GdkPixbuf *thumbnail;
+} PEditorConvData;
+
+static void update_message_area (AppearanceData *data);
+static void create_thumbnail (const gchar *name, GdkPixbuf *default_thumb, AppearanceData *data);
+
 static const gchar *symbolic_names[NUM_SYMBOLIC_COLORS] = {
   "fg_color", "bg_color",
   "text_color", "base_color",
@@ -93,16 +104,22 @@ conv_to_widget_cb (GConfPropertyEditor *peditor, const GConfValue *value)
   {
     GtkListStore *list_store;
     GtkTreeIter iter, sort_iter;
+    PEditorConvData *conv;
 
     list_store = GTK_LIST_STORE (gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (store)));
 
+    g_object_get (peditor, "data", &conv, NULL);
     gtk_list_store_insert_with_values (list_store, &iter, 0,
                                        COL_LABEL, curr_value,
-                                       COL_NAME, curr_value, -1);
+                                       COL_NAME, curr_value,
+                                       COL_THUMBNAIL, conv->thumbnail,
+                                       -1);
     /* convert the tree store iter for use with the sort model */
     gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (store),
                                                     &sort_iter, &iter);
     path = gtk_tree_model_get_string_from_iter (store, &sort_iter);
+
+    create_thumbnail (curr_value, conv->thumbnail, conv->data);
   }
 
   new_value = gconf_value_new (GCONF_VALUE_STRING);
@@ -168,6 +185,91 @@ cursor_theme_sort_func (GtkTreeModel *model,
   return result;
 }
 
+static void
+style_message_area_response_cb (GtkWidget *w,
+                                gint response_id,
+                                AppearanceData *data)
+{
+  GtkSettings *settings = gtk_settings_get_default ();
+  gchar *theme;
+  gchar *engine_path;
+
+  g_object_get (settings, "gtk-theme-name", &theme, NULL);
+  engine_path = gtk_theme_info_missing_engine (theme, FALSE);
+  g_free (theme);
+
+  if (engine_path != NULL) {
+    theme_install_file (GTK_WINDOW (gtk_widget_get_toplevel (data->style_message_area)),
+                        engine_path);
+    g_free (engine_path);
+  }
+  update_message_area (data);
+}
+
+static void
+update_message_area (AppearanceData *data)
+{
+  GtkSettings *settings = gtk_settings_get_default ();
+  gchar *theme = NULL;
+  gchar *engine;
+
+  g_object_get (settings, "gtk-theme-name", &theme, NULL);
+  engine = gtk_theme_info_missing_engine (theme, TRUE);
+  g_free (theme);
+
+  if (data->style_message_area == NULL) {
+    GtkWidget *hbox;
+    GtkWidget *parent;
+    GtkWidget *icon;
+
+    if (engine == NULL)
+      return;
+
+    data->style_message_area = gedit_message_area_new ();
+
+    g_signal_connect (data->style_message_area, "response",
+                      (GCallback) style_message_area_response_cb, data);
+
+    data->style_install_button = gedit_message_area_add_button (
+        GEDIT_MESSAGE_AREA (data->style_message_area),
+        _("Install"), GTK_RESPONSE_APPLY);
+
+    data->style_message_label = gtk_label_new (NULL);
+    gtk_label_set_line_wrap (GTK_LABEL (data->style_message_label), TRUE);
+    gtk_misc_set_alignment (GTK_MISC (data->style_message_label), 0.0, 0.5);
+
+    hbox = gtk_hbox_new (FALSE, 9);
+    icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
+    gtk_misc_set_alignment (GTK_MISC (icon), 0.5, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), data->style_message_label, TRUE, TRUE, 0);
+    gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (data->style_message_area), hbox);
+    gtk_widget_show_all (data->style_message_area);
+    gtk_widget_set_no_show_all (data->style_message_area, TRUE);
+
+    parent = glade_xml_get_widget (data->xml, "gtk_themes_vbox");
+    gtk_box_pack_start (GTK_BOX (parent), data->style_message_area, FALSE, FALSE, 0);
+  }
+
+  if (engine != NULL) {
+    gchar *message = g_strdup_printf (_("This theme will not look as intended because the required GTK+ theme engine '%s' is not installed."),
+                                      engine);
+    gtk_label_set_text (GTK_LABEL (data->style_message_label), message);
+    g_free (message);
+    g_free (engine);
+
+    if (packagekit_available ())
+      gtk_widget_show (data->style_install_button);
+    else
+      gtk_widget_hide (data->style_install_button);
+
+    gtk_widget_show (data->style_message_area);
+    gtk_widget_queue_draw (data->style_message_area);
+  } else {
+    gtk_widget_hide (data->style_message_area);
+  }
+}
+
 static void
 update_color_buttons_from_string (const gchar *color_scheme, AppearanceData *data)
 {
@@ -292,7 +394,7 @@ static void
 style_response_cb (GtkDialog *dialog, gint response_id)
 {
   if (response_id == GTK_RESPONSE_HELP) {
-    /* FIXME: help */
+    capplet_help (GTK_WINDOW (dialog), "goscustdesk-61");
   } else {
     gtk_widget_hide (GTK_WIDGET (dialog));
   }
@@ -318,8 +420,10 @@ gtk_theme_changed (GConfPropertyEditor *peditor,
      * info for the color scheme updates already. */
     g_object_get (settings, "gtk-theme-name", &current, NULL);
 
-    if (strcmp (current, name) != 0)
+    if (strcmp (current, name) != 0) {
       g_object_set (settings, "gtk-theme-name", name, NULL);
+      update_message_area (data);
+    }
 
     g_free (current);
 
@@ -633,8 +737,6 @@ update_thumbnail_in_treeview (const gchar *tv_name,
           COL_THUMBNAIL, theme_thumbnail,
           -1);
   }
-
-  g_object_unref (theme_thumbnail);
 }
 
 static void
@@ -661,6 +763,33 @@ icon_theme_thumbnail_cb (GdkPixbuf *pixbuf,
   update_thumbnail_in_treeview ("icon_themes_list", theme_name, pixbuf, data);
 }
 
+static void
+create_thumbnail (const gchar *name, GdkPixbuf *default_thumb, AppearanceData *data)
+{
+  if (default_thumb == data->icon_theme_icon) {
+    GnomeThemeIconInfo *info;
+    info = gnome_theme_icon_info_find (name);
+    if (info != NULL) {
+      generate_icon_theme_thumbnail_async (info,
+          (ThemeThumbnailFunc) icon_theme_thumbnail_cb, data, NULL);
+    }
+  } else if (default_thumb == data->gtk_theme_icon) {
+    GnomeThemeInfo *info;
+    info = gnome_theme_info_find (name);
+    if (info != NULL && info->has_gtk) {
+      generate_gtk_theme_thumbnail_async (info,
+          (ThemeThumbnailFunc) gtk_theme_thumbnail_cb, data, NULL);
+    }
+  } else if (default_thumb == data->window_theme_icon) {
+    GnomeThemeInfo *info;
+    info = gnome_theme_info_find (name);
+    if (info != NULL && info->has_metacity) {
+      generate_metacity_theme_thumbnail_async (info,
+          (ThemeThumbnailFunc) metacity_theme_thumbnail_cb, data, NULL);
+    }
+  }
+}
+
 static void
 changed_on_disk_cb (GnomeThemeCommonInfo *theme,
 		    GnomeThemeChangeType  change_type,
@@ -740,6 +869,7 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
   GConfValue *value;
   ThumbnailGenFunc generator;
   ThemeThumbnailFunc thumb_cb;
+  PEditorConvData *conv_data;
 
   switch (type)
   {
@@ -832,9 +962,14 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
   gtk_tree_view_column_add_attribute (column, renderer, "text", COL_LABEL);
   gtk_tree_view_append_column (GTK_TREE_VIEW (list), column);
 
+  conv_data = g_new (PEditorConvData, 1);
+  conv_data->data = data;
+  conv_data->thumbnail = thumbnail;
   peditor = gconf_peditor_new_tree_view (NULL, key, list,
       "conv-to-widget-cb", conv_to_widget_cb,
       "conv-from-widget-cb", conv_from_widget_cb,
+      "data", conv_data,
+      "data-free-cb", g_free,
       NULL);
   g_signal_connect (peditor, "value-changed", callback, data);
 
@@ -857,6 +992,9 @@ style_init (AppearanceData *data)
   data->gtk_theme_icon = gdk_pixbuf_new_from_file (GNOMECC_PIXMAP_DIR "/gtk-theme-thumbnailing.png", NULL);
   data->window_theme_icon = gdk_pixbuf_new_from_file (GNOMECC_PIXMAP_DIR "/window-theme-thumbnailing.png", NULL);
   data->icon_theme_icon = gdk_pixbuf_new_from_file (GNOMECC_PIXMAP_DIR "/icon-theme-thumbnailing.png", NULL);
+  data->style_message_area = NULL;
+  data->style_message_label = NULL;
+  data->style_install_button = NULL;
 
   w = glade_xml_get_widget (data->xml, "theme_details");
   g_signal_connect (w, "response", (GCallback) style_response_cb, NULL);
@@ -912,6 +1050,7 @@ style_init (AppearanceData *data)
   g_signal_connect (glade_xml_get_widget (data->xml, "icon_themes_delete"), "clicked", (GCallback) icon_theme_delete_cb, data);
   g_signal_connect (glade_xml_get_widget (data->xml, "cursor_themes_delete"), "clicked", (GCallback) cursor_theme_delete_cb, data);
 
+  update_message_area (data);
   gnome_theme_info_register_theme_change ((ThemeChangedCallback) changed_on_disk_cb, data);
 }
 
