@@ -84,6 +84,7 @@ file_theme_type (const gchar *dir)
 		gboolean match;
 
 		g_file_get_contents (filename, &file_contents, &file_size, NULL);
+		g_free (filename);
 
 		pattern = g_pattern_spec_new ("*[Icon Theme]*");
 		match = g_pattern_match_string (pattern, file_contents);
@@ -158,7 +159,7 @@ transfer_cancel_cb (GtkWidget *dlg, gchar *path)
 	GFile *todelete;
 
 	todelete = g_file_new_for_path (path);
-	g_file_delete (todelete, NULL, NULL);
+	capplet_file_delete_recursive (todelete, NULL);
 
 	g_object_unref (todelete);
 	g_free (path);
