@@ -24,8 +24,8 @@
 #endif
 
 #include <string.h>
-#include <gnome.h>
 #include <glib/gi18n.h>
+#include <stdlib.h>
 
 #include "gconf-property-editor.h"
 #include "gnome-da-capplet.h"
@@ -44,7 +44,7 @@ static void
 close_cb (GtkWidget *window, gint response, gpointer user_data)
 {
     if (response == GTK_RESPONSE_HELP) {
-	capplet_help (GTK_WINDOW (window), "user-guide.xml", "prefs-preferredapps");
+	capplet_help (GTK_WINDOW (window), "prefs-preferredapps");
     }
     else {
 	gtk_widget_destroy (window);
@@ -548,7 +548,7 @@ combo_conv_from_widget (GConfPropertyEditor *peditor, const GConfValue *value)
          * prevent the combo box from jumping back to the previous value if the
          * user has selected Custom */
         gconf_value_set_string (ret, "");
-        return NULL;
+        return ret;
     }
     else
     {
@@ -864,11 +864,10 @@ show_dialog (GnomeDACapplet *capplet, const gchar *start_page)
 int
 main (int argc, char **argv)
 {
-    GnomeProgram *program;
     GnomeDACapplet *capplet;
 
     gchar *start_page = NULL;
-    GOptionContext *option_context;
+    GOptionContext *context;
     GOptionEntry option_entries[] = {
         { "show-page",
           'p',
@@ -881,19 +880,10 @@ main (int argc, char **argv)
         { NULL }
     };
 
-#ifdef ENABLE_NLS
-    bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-    textdomain (GETTEXT_PACKAGE);
-#endif
+    context = g_option_context_new (_("- GNOME Default Applications"));
+    g_option_context_add_main_entries (context, option_entries, GETTEXT_PACKAGE);
 
-    option_context = g_option_context_new (NULL);
-    g_option_context_add_main_entries (option_context, option_entries, GETTEXT_PACKAGE);
-
-    program = gnome_program_init ("gnome-default-applications-properties",
-    				  VERSION, LIBGNOMEUI_MODULE, argc, argv,
-    				  GNOME_PARAM_GOPTION_CONTEXT, option_context,
-				  GNOME_PARAM_NONE);
+    capplet_init (context, &argc, &argv);
 
     glade_init ();
 
@@ -909,7 +899,6 @@ main (int argc, char **argv)
     g_object_unref (capplet->gconf);
 
     gnome_da_xml_free (capplet);
-    g_object_unref (program);
 
     return 0;
 }
