@@ -33,6 +33,8 @@
 #include <X11/Xlib.h>
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-bindings.h>
 
 typedef struct App App;
 typedef struct GrabInfo GrabInfo;
@@ -45,6 +47,8 @@ struct App
     GnomeOutputInfo         *current_output;
 
     GtkWidget	   *dialog;
+    GtkWidget      *current_monitor_event_box;
+    GtkWidget      *current_monitor_label;
     GtkListStore   *resolution_store;
     GtkWidget	   *resolution_combo;
     GtkWidget	   *refresh_combo;
@@ -60,17 +64,20 @@ struct App
 
 static void rebuild_gui (App *app);
 static void on_rate_changed (GtkComboBox *box, gpointer data);
+static gboolean output_overlaps (GnomeOutputInfo *output, GnomeRRConfig *config);
+static void select_current_output_from_dialog_position (App *app);
 
 static void
-show_error (const char *err)
+error_message (App *app, const char *primary_text, const char *secondary_text)
 {
-    GtkWidget *dialog = gtk_message_dialog_new (
-	NULL,
-	GTK_DIALOG_DESTROY_WITH_PARENT,
-	GTK_MESSAGE_WARNING,
-	GTK_BUTTONS_OK, "%s", err);
+    GtkWidget *dialog;
 
-    gtk_window_set_title (GTK_WINDOW (dialog), "");
+    dialog = gtk_message_dialog_new ((app && app->dialog) ? GTK_WINDOW (app->dialog) : NULL,
+				     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
+				     GTK_MESSAGE_ERROR,
+				     GTK_BUTTONS_CLOSE,
+				     "%s", primary_text);
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", secondary_text);
 
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
@@ -107,7 +114,6 @@ on_screen_changed (GnomeRRScreen *scr,
     GnomeRRConfig *current;
     App *app = data;
     int i;
-    GnomeOutputInfo *best;
 
     current = gnome_rr_config_new_current (app->screen);
 
@@ -151,25 +157,7 @@ on_screen_changed (GnomeRRScreen *scr,
     }
 #endif
 
-    /* Select an output */
-    best = NULL;
-    for (i = 0; app->current_configuration->outputs[i] != NULL; ++i)
-    {
-	GnomeOutputInfo *output = app->current_configuration->outputs[i];
-
-	if (output->connected)
-	{
-	    char *cur_name =
-		app->current_output? app->current_output->name : NULL;
-
-	    if ((cur_name && strcmp (output->name, cur_name) == 0) || !best)
-		best = output;
-	}
-    }
-
-    app->current_output = best;
-
-    rebuild_gui (app);
+    select_current_output_from_dialog_position (app);
 }
 
 static void
@@ -353,7 +341,8 @@ rebuild_rotation_combo (App *app)
 
 	app->current_output->rotation = info->rotation;
 
-	if (gnome_rr_config_applicable (app->current_configuration, app->screen))
+	/* NULL-GError --- FIXME: we should say why this rotation is not available! */
+	if (gnome_rr_config_applicable (app->current_configuration, app->screen, NULL))
 	{
  	    add_key (app->rotation_combo, info->name, 0, 0, 0, info->rotation);
 
@@ -441,6 +430,39 @@ count_all_outputs (GnomeRRConfig *config)
 }
 #endif
 
+static void
+rebuild_current_monitor_label (App *app)
+{
+	char *str;
+	gboolean free_str;
+	GdkColor color;
+	gboolean use_color;
+
+	if (app->current_output)
+	{
+	    str = g_strdup_printf (_("<b>Monitor: %s</b>"), app->current_output->display_name);
+	    free_str = TRUE;
+	    gnome_rr_labeler_get_color_for_output (app->labeler, app->current_output, &color);
+	    use_color = TRUE;
+	}
+	else
+	{
+	    str = _("<b>Monitor</b>");
+	    free_str = FALSE;
+	    use_color = FALSE;
+	}
+
+	gtk_label_set_markup (GTK_LABEL (app->current_monitor_label), str);
+
+	if (free_str)
+	    g_free (str);
+
+	if (use_color)
+	    gtk_widget_modify_bg (app->current_monitor_event_box, app->current_monitor_event_box->state, &color);
+
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (app->current_monitor_event_box), use_color);
+}
+
 static void
 rebuild_resolution_combo (App *app)
 {
@@ -515,6 +537,7 @@ rebuild_gui (App *app)
     g_debug ("rebuild gui, is on: %d", app->current_output->on);
 #endif
 
+    rebuild_current_monitor_label (app);
     rebuild_resolution_combo (app);
     rebuild_rate_combo (app);
     rebuild_rotation_combo (app);
@@ -644,6 +667,47 @@ on_resolution_changed (GtkComboBox *box, gpointer data)
     foo_scroll_area_invalidate (FOO_SCROLL_AREA (app->area));
 }
 
+static void
+lay_out_outputs_horizontally (App *app)
+{
+    int i;
+    int x;
+
+    /* Lay out all the monitors horizontally when "mirror screens" is turned
+     * off, to avoid having all of them overlapped initially.  We put the
+     * outputs turned off on the right-hand side.
+     */
+
+    x = 0;
+
+    /* First pass, all "on" outputs */
+
+    for (i = 0; app->current_configuration->outputs[i]; ++i)
+    {
+	GnomeOutputInfo *output;
+
+	output = app->current_configuration->outputs[i];
+	if (output->connected && output->on)
+	    output->x = x;
+
+	x += output->width;
+    }
+
+    /* Second pass, all the black screens */
+
+    for (i = 0; app->current_configuration->outputs[i]; ++i)
+    {
+	GnomeOutputInfo *output;
+
+	output = app->current_configuration->outputs[i];
+	if (!(output->connected && output->on))
+	    output->x = x;
+
+	x += output->width;
+    }
+    
+}
+
 static void
 on_clone_changed (GtkWidget *box, gpointer data)
 {
@@ -665,6 +729,11 @@ on_clone_changed (GtkWidget *box, gpointer data)
 	    }
 	}
     }
+    else
+    {
+	if (output_overlaps (app->current_output, app->current_configuration))
+	    lay_out_outputs_horizontally (app);
+    }
 
     rebuild_gui (app);
 }
@@ -1483,20 +1552,6 @@ make_text_combo (GtkWidget *widget, int sort_column)
     }
 }
 
-static Atom
-gnome_randr_atom (void)
-{
-    static Atom atom = None;
-
-    if (!atom)
-    {
-	atom = XInternAtom (gdk_x11_get_default_xdisplay(),
-			    "_GNOME_RANDR_ATOM", FALSE);
-    }
-
-    return atom;
-}
-
 static void
 compute_virtual_size_for_configuration (GnomeRRConfig *config, int *ret_width, int *ret_height)
 {
@@ -1554,7 +1609,9 @@ check_required_virtual_size (App *app)
 static void
 apply (App *app)
 {
-    GError *err = NULL;
+    GError *error = NULL;
+    DBusGConnection *connection;
+    DBusGProxy *proxy;
 
     gnome_rr_config_sanitize (app->current_configuration);
 
@@ -1562,23 +1619,37 @@ apply (App *app)
 
     foo_scroll_area_invalidate (FOO_SCROLL_AREA (app->area));
 
-    if (gnome_rr_config_save (app->current_configuration, &err))
+    if (!gnome_rr_config_save (app->current_configuration, &error))
     {
-	XEvent message;
-
-	message.xclient.type = ClientMessage;
-	message.xclient.message_type = gnome_randr_atom();
-	message.xclient.format = 8;
-
-#if 0
-	g_debug ("Sending client message");
-#endif
-
-	XSendEvent (gdk_x11_get_default_xdisplay(),
-		    gdk_x11_get_default_root_xwindow(),
-		    FALSE,
-		    StructureNotifyMask, &message);
+	error_message (app, _("Could not save the monitor configuration"), error->message);
+	g_error_free (error);
+	return;
     }
+
+    connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+    if (connection == NULL) {
+	error_message (app, _("Could not get session bus while applying display configuration"), error->message);
+	g_error_free (error);
+	return;
+    }
+
+    proxy = dbus_g_proxy_new_for_name (connection,
+				       "org.gnome.SettingsDaemon",
+				       "/org/gnome/SettingsDaemon/XRANDR",
+				       "org.gnome.SettingsDaemon.XRANDR");
+    if (!proxy) {
+	error_message (app, _("Could not get org.gnome.SettingsDaemon.XRANDR"), NULL);
+	return;
+
+    }
+
+    if (!dbus_g_proxy_call (proxy, "ApplyConfiguration", &error, G_TYPE_INVALID, G_TYPE_INVALID)) {
+	error_message (app, _("Could not apply the selected configuration"), error->message);
+	g_error_free (error);
+    }
+
+    g_object_unref (proxy);
+    dbus_g_connection_unref (connection);
 }
 
 #if 0
@@ -1608,8 +1679,15 @@ static void
 on_detect_displays (GtkWidget *widget, gpointer data)
 {
     App *app = data;
+    GError *error;
 
-    gnome_rr_screen_refresh (app->screen);
+    error = NULL;
+    if (!gnome_rr_screen_refresh (app->screen, &error)) {
+	if (error) {
+	    error_message (app, _("Could not detect displays"), error->message);
+	    g_error_free (error);
+	}
+    }
 }
 
 #define SHOW_ICON_KEY "/apps/gnome_settings_daemon/xrandr/show_notification_icon"
@@ -1625,6 +1703,162 @@ on_show_icon_toggled (GtkWidget *widget, gpointer data)
 			   gtk_toggle_button_get_active (tb), NULL);
 }
 
+static GnomeOutputInfo *
+get_nearest_output (GnomeRRConfig *configuration, int x, int y)
+{
+    int i;
+    int nearest_index;
+    int nearest_dist;
+
+    nearest_index = -1;
+    nearest_dist = G_MAXINT;
+
+    for (i = 0; configuration->outputs[i] != NULL; i++)
+    {
+	GnomeOutputInfo *output;
+	int dist_x, dist_y;
+
+	output = configuration->outputs[i];
+
+	if (!(output->connected && output->on))
+	    continue;
+
+	if (x < output->x)
+	    dist_x = output->x - x;
+	else if (x >= output->x + output->width)
+	    dist_x = x - (output->x + output->width) + 1;
+	else
+	    dist_x = 0;
+
+	if (y < output->y)
+	    dist_y = output->y - y;
+	else if (y >= output->y + output->height)
+	    dist_y = y - (output->y + output->height) + 1;
+	else
+	    dist_y = 0;
+
+	if (MIN (dist_x, dist_y) < nearest_dist)
+	{
+	    nearest_dist = MIN (dist_x, dist_y);
+	    nearest_index = i;
+	}
+    }
+
+    if (nearest_index != -1)
+	return configuration->outputs[nearest_index];
+    else
+	return NULL;
+		
+}
+
+/* Gets the output that contains the largest intersection with the window.
+ * Logic stolen from gdk_screen_get_monitor_at_window().
+ */
+static GnomeOutputInfo *
+get_output_for_window (GnomeRRConfig *configuration, GdkWindow *window)
+{
+    GdkRectangle win_rect;
+    int i;
+    int largest_area;
+    int largest_index;
+
+    gdk_window_get_geometry (window, &win_rect.x, &win_rect.y, &win_rect.width, &win_rect.height, NULL);
+    gdk_window_get_origin (window, &win_rect.x, &win_rect.y);
+
+    largest_area = 0;
+    largest_index = -1;
+
+    for (i = 0; configuration->outputs[i] != NULL; i++)
+    {
+	GnomeOutputInfo *output;
+	GdkRectangle output_rect, intersection;
+
+	output = configuration->outputs[i];
+
+	output_rect.x	   = output->x;
+	output_rect.y	   = output->y;
+	output_rect.width  = output->width;
+	output_rect.height = output->height;
+
+	if (output->connected && output->on && gdk_rectangle_intersect (&win_rect, &output_rect, &intersection))
+	{
+	    int area;
+
+	    area = intersection.width * intersection.height;
+	    if (area > largest_area)
+	    {
+		largest_area = area;
+		largest_index = i;
+	    }
+	}
+    }
+
+    if (largest_index != -1)
+	return configuration->outputs[largest_index];
+    else
+	return get_nearest_output (configuration,
+				   win_rect.x + win_rect.width / 2,
+				   win_rect.y + win_rect.height / 2);
+}
+
+/* We select the current output, i.e. select the one being edited, based on
+ * which output is showing the configuration dialog.
+ */
+static void
+select_current_output_from_dialog_position (App *app)
+{
+    GnomeRROutput *output;
+
+    if (GTK_WIDGET_REALIZED (app->dialog))
+    {
+	output = get_output_for_window (app->current_configuration, app->dialog->window);
+
+	if (output)
+	    app->current_output = output;
+    }
+
+    rebuild_gui (app);
+}
+
+/* This is a GtkWidget::map-event handler.  We wait for the display-properties
+ * dialog to be mapped, and then we select the output which corresponds to the
+ * monitor on which the dialog is being shown.
+ */
+static gboolean
+dialog_map_event_cb (GtkWidget *widget, GdkEventAny *event, gpointer data)
+{
+    App *app = data;
+
+    select_current_output_from_dialog_position (app);
+    return FALSE;
+}
+
+static void
+hide_help_button (App *app)
+{
+    GtkWidget *action_area;
+    GList *children;
+    GList *l;
+
+    action_area = gtk_dialog_get_action_area (GTK_DIALOG (app->dialog));
+    children = gtk_container_get_children (GTK_CONTAINER (action_area));
+
+    for (l = children; l; l = l->next)
+    {
+	GtkWidget *child;
+	int response;
+
+	child = GTK_WIDGET (l->data);
+
+	response = gtk_dialog_get_response_for_widget (GTK_DIALOG (app->dialog), child);
+	if (response == GTK_RESPONSE_HELP)
+	{
+	    gtk_widget_hide (child);
+	    return;
+	}
+    }
+}
+
 static void
 run_application (App *app)
 {
@@ -1634,6 +1868,7 @@ run_application (App *app)
 #define GLADE_FILE GLADEDIR "/display-capplet.glade"
     GladeXML *xml;
     GtkWidget *align;
+    GError *error;
 
     xml = glade_xml_new (GLADE_FILE, NULL, NULL);
     if (!xml)
@@ -1642,12 +1877,13 @@ run_application (App *app)
 	return;
     }
 
+    error = NULL;
     app->screen = gnome_rr_screen_new (gdk_screen_get_default (),
-				       on_screen_changed, app);
+				       on_screen_changed, app, &error);
     if (!app->screen)
     {
-	g_warning (_("The X server does not support the XRANDR extension.  Runtime resolution changes to the display size are not available."));
-	show_error (_("The X server does not support the XRANDR extension.  Runtime resolution changes to the display size are not available."));
+	error_message (NULL, _("Could not get screen information"), error->message);
+	g_error_free (error);
 	g_object_unref (xml);
 	return;
     }
@@ -1655,11 +1891,16 @@ run_application (App *app)
     app->client = gconf_client_get_default ();
 
     app->dialog = glade_xml_get_widget (xml, "dialog");
+    g_signal_connect_after (app->dialog, "map-event",
+			    G_CALLBACK (dialog_map_event_cb), app);
 
     gtk_window_set_default_icon_name ("gnome-display-properties");
     gtk_window_set_icon_name (GTK_WINDOW (app->dialog),
 			      "gnome-display-properties");
 
+    app->current_monitor_event_box = glade_xml_get_widget (xml, "current_monitor_event_box");
+    app->current_monitor_label = glade_xml_get_widget (xml, "current_monitor_label");
+
     app->resolution_combo = glade_xml_get_widget (xml, "resolution_combo");
     g_signal_connect (app->resolution_combo, "changed",
 		      G_CALLBACK (on_resolution_changed), app);
@@ -1711,6 +1952,9 @@ run_application (App *app)
 
     gtk_container_add (GTK_CONTAINER (align), app->area);
 
+    /* Until we have help to show, we'll just hide the Help button */
+    hide_help_button (app);
+
     on_screen_changed (app->screen, app);
 
     g_object_unref (xml);
